@@ -9,9 +9,10 @@ import logging
 from bs4 import BeautifulSoup, Tag
 from brow.interface.selenium import FirefoxBrowser as FullBrowser
 from brow.interface.simple import SimpleFirefoxBrowser as SimpleBrowser
+from brow.utils import Soup
 
 from .compat import *
-from .exception import RobotError
+from .exception import RobotError, ParseError
 from . import environ
 
 
@@ -29,9 +30,12 @@ class BaseAmazon(object):
         # docs: https://www.crummy.com/software/BeautifulSoup/bs4/doc/
         # bs4 codebase: http://bazaar.launchpad.net/~leonardr/beautifulsoup/bs4/files
         if isinstance(body, Tag): return body
-        soup = BeautifulSoup(body, "html.parser")
+        soup = Soup(body)
         return soup
 
+    def dump(self):
+        return self.soup.prettify()
+
 
 class WishlistElement(BaseAmazon):
     """Wishlist.get() returns an instance of this object"""
@@ -55,7 +59,7 @@ class WishlistElement(BaseAmazon):
         uuid = ""
         a_url = self.url
         if a_url:
-            m = re.search("/dp/([^/]+)", self.url)
+            m = re.search("/dp/([^/\?\&]+)", self.url)
             if m:
                 uuid = m.group(1)
         else:
@@ -80,9 +84,11 @@ class WishlistElement(BaseAmazon):
         # http://stackoverflow.com/a/2832635/5006
         el = self.soup.find("a", id=re.compile("^itemName_"))
         if el and ("href" in el.attrs):
-            m = re.search("/dp/([^/]+)", el.attrs["href"])
-            if m:
-                href = "{}/dp/{}/".format(self.host, m.group(1))
+            href = self.host + el.attrs["href"]
+
+#             m = re.search("/dp/([^/]+)", el.attrs["href"])
+#             if m:
+#                 href = "{}/dp/{}/".format(self.host, m.group(1))
 
         return href
 
@@ -130,12 +136,16 @@ class WishlistElement(BaseAmazon):
 
     @property
     def price(self):
-        el = self.soup.find("span", id=re.compile("^itemPrice_"))
-        if not el or len(el.contents) < 1:
-            return 0.0
-
         price = 0.0
-        try:
+
+        el = self.soup.find("span", id=re.compile(r"^itemPrice_"))
+        #pout.v(el.prettify())
+        #pout.v(self.soup.prettify())
+#         pout.v(str(self.soup))
+#         import testdata
+#         f = testdata.create_file("output.html", self.soup.prettify())
+#         pout.v(f)
+        if el and len(el.contents) >= 1:
             # the new HTML actually has separate spans for whole currency
             # units and fractional currency units
             try:
@@ -150,11 +160,26 @@ class WishlistElement(BaseAmazon):
                 price = float(whole) + (float(fract) / 100.0)
 
             except AttributeError:
-                s = "".join(el.strings).split("-")[0].strip()
-                price = float(s.lstrip('$').replace(",", ""))
+                try:
+                    s = "".join(el.strings).split("-")[0].strip()
+                    price = float(s.lstrip('$').replace(",", ""))
+                except ValueError:
+                    price = 0.0
 
-        except (ValueError, IndexError):
-            logger.error('Unable to parse price span: %s', el)
+#         else:
+#             # 6-18-2020, I have no idea what this code is for anymore
+#             in_stock = True
+#             el_available = self.soup.find("div", class_="itemAvailability")
+#             if el_available:
+#                 if el_available.find("span", class_="itemAvailMessage"):
+#                     if el_available.find("a", class_="itemAvailSignup"):
+#                         in_stock = True
+# 
+#             if not in_stock:
+#                 raise ParseError(
+#                     msg="Could not find price for {}".format(self.title),
+#                     body=self.body
+#                 )
 
         return price
 
@@ -214,27 +239,33 @@ class WishlistElement(BaseAmazon):
         ret = None
         format_str = '%B %d, %Y'
         el = self.soup.find('span', id=re.compile('^itemAddedDate_'))
-        if el is None or len(el.contents) < 3:
-            el = self.soup.select_one(".dateAddedText > span")
-            if el:
-                s = el.get_text().strip()
-                while s:
-                    try:
-                        ret = datetime.datetime.strptime(s, format_str)
-                        break
 
-                    except ValueError:
-                        bits = s.split(" ", 1)
-                        if len(bits) > 1:
-                            s = " ".join(bits[1:])
-                        else:
-                            s = ""
-
-                if not ret:
-                    logger.error('Unable to find added date for item.')
+        if el:
+            for content in el.contents:
+                m = re.search(r"[^\d\s]+\s+\d{1,2},?\s\d{4}", content)
+                if m:
+                    ret = datetime.datetime.strptime(m.group(0).strip(), format_str).date()
+                    break
 
         else:
-            ret = datetime.datetime.strptime(el.contents[2].strip(), format_str)
+            if el is None or len(el.contents) < 3:
+                el = self.soup.select_one(".dateAddedText > span")
+                if el:
+                    s = el.get_text().strip()
+                    while s:
+                        try:
+                            ret = datetime.datetime.strptime(s, format_str).date()
+                            break
+
+                        except ValueError:
+                            bits = s.split(" ", 1)
+                            if len(bits) > 1:
+                                s = " ".join(bits[1:])
+                            else:
+                                s = ""
+
+                    if not ret:
+                        logger.error('Unable to find added date for item.')
 
         return ret
 
@@ -254,15 +285,31 @@ class WishlistElement(BaseAmazon):
 
         :returns: tuple of ints (wanted, has)
         """
+        ret = None
         el = self.soup.find(id=re.compile("^itemQuantityRow_"))
         bits = [s for s in el.stripped_strings]
-        return (int(bits[1]), int(bits[3]))
+        if len(bits) == 4:
+            ret = (int(bits[1]), int(bits[3]))
+
+        elif len(bits) == 2:
+            ret = (0, 0)
+
+        else:
+            raise ParseError(
+                msg="Could not find quantity for {}".format(self.title),
+                body=self.body
+            )
+
+        return ret
 
     @property
     def source(self):
         """Return "amazon" if product is offered by amazon, otherwise return "marketplace" """
-        ret = "marketplace"
-        if not self.is_digital():
+        if self.is_digital():
+            ret = "amazon"
+
+        else:
+            ret = "marketplace"
             el = self.soup.find(class_=re.compile("^itemAvailOfferedBy"))
             if el:
                 s = el.string
@@ -275,13 +322,24 @@ class WishlistElement(BaseAmazon):
     def body(self):
         return self.soup.prettify()
 
-    def __init__(self, element, page_url=""):
+    @property
+    def page_url(self):
+        ret = self._page_url
+        if ret:
+            el = self.soup.find(id=re.compile("^itemMain_"))
+            if el:
+                ret += "#{}".format(el.attrs["id"])
+        return ret
+
+    def __init__(self, element, page_url="", page=0):
         """
         :param element: mixed, the html for the element
         :param page_url: string, the current page url
+        :param page: int, the current page number
         """
         self.soup = self.soupify(element)
-        self.page_url = page_url
+        self._page_url = page_url
+        self.page = int(page)
 
     def is_digital(self):
         """Return true if this is a digital good like a Kindle book or mp3"""
@@ -297,8 +355,16 @@ class WishlistElement(BaseAmazon):
                 if not re.search(r"auto-delivered\s+wirelessly", s, re.I):
                     if not re.search(r"amazon\s+digital\s+services", s, re.I):
                         ret = False
+
+        else:
+            ret = "kindle edition" in self.author.lower()
+
         return ret
 
+    def in_stock(self):
+        """Return True if the item is in stock"""
+        return self.price or self.is_digital()
+
     def is_amazon(self):
         """returns True if product is offered by amazon, otherwise False"""
         return "amazon" in self.source
@@ -314,7 +380,12 @@ class WishlistElement(BaseAmazon):
         json_item["marketplace_price"] = self.marketplace_price
         json_item["comment"] = self.comment
         json_item["author"] = self.author
-        json_item["added"] = self.added.strftime('%B %d, %Y')
+
+        json_item["added"] = "UNKNOWN"
+        added = self.added
+        if added:
+            json_item["added"] = added.strftime('%B %d, %Y')
+
         json_item["rating"] = self.rating
         json_item["quantity"] = {
             "wanted": self.wanted_count,
@@ -353,25 +424,28 @@ class Wishlist(BaseAmazon):
             path = "/gp/registry/wishlist/{}".format(name)
         return "{}/{}".format(self.host.rstrip("/"), path.lstrip("/"))
 
-    def get_items(self, soup, current_page_url):
+    def get_items(self, soup, current_page_url, current_page=0):
         """this will return the wishlist elements on the current page"""
         html_items = soup.findAll("div", {"id": re.compile("^item_")})
         for i, html_item in enumerate(html_items):
-            item = self.element_class(html_item, current_page_url)
+            item = self.element_class(html_item, current_page_url, current_page)
             yield item
 
     def __iter__(self):
         host = self.host
         # so the lists are circular for some reason, so we need to track what pages
         # we have seen and stop when we see the item uuid again
+        name = self.name
         seen_uuids = set()
         url = self.get_wishlist_url()
         with SimpleBrowser.session() as b:
+            page = 1
             while url:
                 b.load(url)
+                b.dump(basename="{}-{}".format(name, page))
                 soup = b.soup
 
-                for item in self.get_items(soup, url):
+                for item in self.get_items(soup, url, page):
                     yield item
 
                 url = ""
@@ -381,6 +455,8 @@ class Wishlist(BaseAmazon):
                     if uuid:
                         elem = soup.select_one("input#sort-by-price-load-more-items-url")
                         if uuid not in seen_uuids:
+                            logger.debug("First time seeing uuid {}".format(uuid))
                             url = self.get_wishlist_url(elem["value"])
                             seen_uuids.add(uuid)
+                            page += 1
 
